@@ -54,7 +54,6 @@
 #include "gpm-manager.h"
 #include "gpm-screensaver.h"
 #include "gpm-backlight.h"
-#include "gpm-screensaver.h"
 #include "gpm-session.h"
 #include "gpm-stock-icons.h"
 #include "gpm-prefs-server.h"
@@ -379,7 +378,7 @@ gpm_manager_notify (GpmManager *manager, NotifyNotification **notification_class
 	gpm_manager_notify_close (manager, *notification_class);
 
 	/* if the status icon is hidden, don't point at it */
-	if (gtk_status_icon_get_visible (manager->priv->status_icon))
+	if (gtk_status_icon_is_embedded (manager->priv->status_icon))
 		notification = notify_notification_new_with_status_icon (title, message, icon, manager->priv->status_icon);
 	else
 		notification = notify_notification_new (title, message, icon, NULL);
@@ -416,39 +415,120 @@ out:
 	return ret;
 }
 
+
+/**
+ * gpm_manager_sleep_failure_response_cb:
+ **/
+static void
+gpm_manager_sleep_failure_response_cb (GtkDialog *dialog, gint response_id, GpmManager *manager)
+{
+	GdkScreen *screen;
+	GtkWidget *dialog_error;
+	GError *error = NULL;
+	gboolean ret;
+	gchar *uri = NULL;
+
+	/* user clicked the help button */
+	if (response_id == GTK_RESPONSE_HELP) {
+		uri = gconf_client_get_string (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED_URI, NULL);
+		screen = gdk_screen_get_default();
+		ret = gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
+		if (!ret) {
+			dialog_error = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
+							       "Failed to show uri %s", error->message);
+			gtk_dialog_run (GTK_DIALOG (dialog_error));
+			g_error_free (error);
+		}
+	}
+
+	gtk_widget_destroy (GTK_WIDGET (dialog));
+	g_free (uri);
+}
+
+/**
+ * gpm_manager_sleep_failure:
+ **/
+static void
+gpm_manager_sleep_failure (GpmManager *manager, gboolean is_suspend, const gchar *detail)
+{
+	gboolean show_sleep_failed;
+	GString *string = NULL;
+	const gchar *title;
+	gchar *uri = NULL;
+	const gchar *icon;
+	GtkWidget *dialog;
+
+	/* only show this if specified in gconf */
+	show_sleep_failed = gconf_client_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED, NULL);
+
+	egg_debug ("sleep failed");
+	gpm_manager_play (manager, GPM_MANAGER_SOUND_SUSPEND_ERROR, TRUE);
+
+	/* only emit if in GConf */
+	if (!show_sleep_failed)
+		goto out;
+
+	/* TRANSLATORS: window title: there was a problem putting the machine to sleep */
+	string = g_string_new ("");
+	if (is_suspend) {
+		/* TRANSLATORS: message text */
+		g_string_append (string, _("Your computer failed to suspend."));
+		/* TRANSLATORS: title text */
+		title = _("Failed to suspend");
+		icon = GPM_STOCK_SUSPEND;
+	} else {
+		/* TRANSLATORS: message text */
+		g_string_append (string, _("Your computer failed to hibernate."));
+		/* TRANSLATORS: title text */
+		title = _("Failed to hibernate");
+		icon = GPM_STOCK_HIBERNATE;
+	}
+
+	/* TRANSLATORS: message text */
+	g_string_append_printf (string, "\n\n%s %s", _("The failure was reported as:"), detail);
+
+	/* show modal dialog */
+	dialog = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+						     GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+						     "<span size='larger'><b>%s</b></span>", title);
+	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), "%s", string->str);
+	gtk_window_set_icon_name (GTK_WINDOW(dialog), icon);
+
+	/* show a button? */
+	uri = gconf_client_get_string (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED_URI, NULL);
+	if (uri != NULL && uri[0] != '\0') {
+		/* TRANSLATORS: button text, visit the suspend help website */
+		gtk_dialog_add_button (GTK_DIALOG (dialog), _("Visit help page"), GTK_RESPONSE_HELP);
+	}
+
+	/* wait async for close */
+	gtk_widget_show (dialog);
+	g_signal_connect (dialog, "response", G_CALLBACK (gpm_manager_sleep_failure_response_cb), manager);
+out:
+	g_free (uri);
+	g_string_free (string, TRUE);
+}
+
 /**
  * gpm_manager_action_suspend:
  **/
 static gboolean
 gpm_manager_action_suspend (GpmManager *manager, const gchar *reason)
 {
-	gboolean allowed;
+	gboolean ret;
 	GError *error = NULL;
 
-	/* check if the admin has disabled */
-	allowed = gconf_client_get_bool (manager->priv->conf, GPM_CONF_CAN_SUSPEND, NULL);
-	if (allowed == FALSE) {
-		/* error msg as disabled in gconf */
-		gpm_manager_notify (manager, &manager->priv->notification,
-				    /* TRANSLATORS: the action was not done */
-				    _("Action disallowed"),
-				    /* TRANSLATORS: admin has disabled ability to do this */
-				    _("Suspend support has been disabled. Contact your administrator for more details."),
-				    GPM_MANAGER_NOTIFY_TIMEOUT_SHORT,
-				    GPM_STOCK_APP_ICON,
-				    NOTIFY_URGENCY_NORMAL);
-		return FALSE;
-	}
-
 	/* check to see if we are inhibited */
 	if (gpm_manager_is_inhibit_valid (manager, FALSE, "suspend") == FALSE)
 		return FALSE;
 
 	egg_debug ("suspending, reason: %s", reason);
-	gpm_control_suspend (manager->priv->control, &error);
-	gpm_button_reset_time (manager->priv->button);
-	if (error != NULL)
+	ret = gpm_control_suspend (manager->priv->control, &error);
+	if (!ret) {
+		gpm_manager_sleep_failure (manager, TRUE, error->message);
 		g_error_free (error);
+	}
+	gpm_button_reset_time (manager->priv->button);
 	return TRUE;
 }
 
@@ -458,33 +538,20 @@ gpm_manager_action_suspend (GpmManager *manager, const gchar *reason)
 static gboolean
 gpm_manager_action_hibernate (GpmManager *manager, const gchar *reason)
 {
-	gboolean allowed;
+	gboolean ret;
 	GError *error = NULL;
 
-	/* check if the admin has disabled */
-	allowed = gconf_client_get_bool (manager->priv->conf, GPM_CONF_CAN_HIBERNATE, NULL);
-	if (allowed == FALSE) {
-		/* error msg as disabled in gconf */
-		gpm_manager_notify (manager, &manager->priv->notification,
-				    /* TRANSLATORS: the action was not done */
-				    _("Action disallowed"),
-				    /* TRANSLATORS: admin has disabled ability to do this */
-				    _("Hibernate support has been disabled. Contact your administrator for more details."),
-				    GPM_MANAGER_NOTIFY_TIMEOUT_SHORT,
-				    GPM_STOCK_APP_ICON,
-				    NOTIFY_URGENCY_NORMAL);
-		return FALSE;
-	}
-
 	/* check to see if we are inhibited */
 	if (gpm_manager_is_inhibit_valid (manager, FALSE, "hibernate") == FALSE)
 		return FALSE;
 
 	egg_debug ("hibernating, reason: %s", reason);
-	gpm_control_hibernate (manager->priv->control, &error);
-	gpm_button_reset_time (manager->priv->button);
-	if (error != NULL)
+	ret = gpm_control_hibernate (manager->priv->control, &error);
+	if (!ret) {
+		gpm_manager_sleep_failure (manager, TRUE, error->message);
 		g_error_free (error);
+	}
+	gpm_button_reset_time (manager->priv->button);
 	return TRUE;
 }
 
@@ -792,11 +859,11 @@ gpm_manager_button_pressed_cb (GpmButton *button, const gchar *type, GpmManager
 /**
  * gpm_manager_get_spindown_timeout:
  **/
-static guint
+static gint
 gpm_manager_get_spindown_timeout (GpmManager *manager)
 {
 	gboolean enabled;
-	guint timeout;
+	gint timeout;
 
 	/* get policy */
 	if (!manager->priv->on_battery) {
@@ -818,7 +885,7 @@ static void
 gpm_manager_client_changed_cb (DkpClient *client, GpmManager *manager)
 {
 	gboolean event_when_closed;
-	guint timeout;
+	gint timeout;
 	gboolean on_battery;
 
 	/* get the on-battery state */
@@ -927,34 +994,7 @@ gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
 		gpm_manager_sync_policy_sleep (manager);
 }
 
-/**
- * gpm_manager_tray_icon_hibernate:
- * @manager: This class instance
- * @tray: The tray object
- *
- * The icon tray hibernate callback, which only should happen if both policy and
- * the inhibit states are valid.
- **/
-static void
-gpm_manager_tray_icon_hibernate (GpmManager *manager, GpmTrayIcon *tray)
-{
-	gpm_manager_action_hibernate (manager, "User clicked on tray");
-}
-
-/**
- * gpm_manager_tray_icon_suspend:
- * @manager: This class instance
- * @tray: The tray object
- *
- * The icon tray suspend callback, which only should happen if both policy and
- * the inhibit states are valid.
- **/
-static void
-gpm_manager_tray_icon_suspend (GpmManager *manager, GpmTrayIcon *tray)
-{
-	gpm_manager_action_suspend (manager, "User clicked on tray");
-}
-
+#if 0
 /**
  * gpm_manager_screensaver_auth_request_cb:
  * @manager: This manager class instance
@@ -979,6 +1019,7 @@ gpm_manager_screensaver_auth_request_cb (GpmScreensaver *screensaver, gboolean a
 		}
 	}
 }
+#endif
 
 /**
  * gpm_manager_perhaps_recall_response_cb:
@@ -1201,8 +1242,9 @@ gpm_manager_engine_discharging_cb (GpmEngine *engine, DkpDevice *device, GpmMana
 	const gchar *message;
 	gdouble percentage;
 	gint64 time_to_empty;
-	gchar *remaining_text;
+	gchar *remaining_text = NULL;
 	gchar *icon = NULL;
+	const gchar *type_desc;
 
 	/* only action this if specified in gconf */
 	ret = gconf_client_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_DISCHARGING, NULL);
@@ -1217,20 +1259,36 @@ gpm_manager_engine_discharging_cb (GpmEngine *engine, DkpDevice *device, GpmMana
 		      "percentage", &percentage,
 		      "time-to-empty", &time_to_empty,
 		      NULL);
-	remaining_text = gpm_get_timestring (time_to_empty);
+
+	/* only show text if there is a valid time */
+	if (time_to_empty > 0)
+		remaining_text = gpm_get_timestring (time_to_empty);
+	type_desc = gpm_device_type_to_localised_text (type, 1);
 
 	if (type == DKP_DEVICE_TYPE_BATTERY) {
 		/* TRANSLATORS: laptop battery is now discharging */
 		title = _("Battery Discharging");
 
-		/* TRANSLATORS: tell the user how much time they have got */
-		message = g_strdup_printf (_("%s of battery power remaining (%.1f%%)"), remaining_text, percentage);
+		if (remaining_text != NULL) {
+			/* TRANSLATORS: tell the user how much time they have got */
+			message = g_strdup_printf (_("%s of battery power remaining (%.1f%%)"), remaining_text, percentage);
+		} else {
+			/* TRANSLATORS: the device is discharging, but we only have a percentage */
+			message = g_strdup_printf (_("%s discharging (%.1f%%)"),
+						   type_desc, percentage);
+		}
 	} else if (type == DKP_DEVICE_TYPE_UPS) {
 		/* TRANSLATORS: UPS is now discharging */
 		title = _("UPS Discharging");
 
-		/* TRANSLATORS: tell the user how much time they have got */
-		message = g_strdup_printf (_("%s of UPS backup power remaining (%.1f%%)"), remaining_text, percentage);
+		if (remaining_text != NULL) {
+			/* TRANSLATORS: tell the user how much time they have got */
+			message = g_strdup_printf (_("%s of UPS backup power remaining (%.1f%%)"), remaining_text, percentage);
+		} else {
+			/* TRANSLATORS: the device is discharging, but we only have a percentage */
+			message = g_strdup_printf (_("%s discharging (%.1f%%)"),
+						   type_desc, percentage);
+		}
 	} else {
 		/* nothing else of interest */
 		goto out;
@@ -1240,97 +1298,12 @@ gpm_manager_engine_discharging_cb (GpmEngine *engine, DkpDevice *device, GpmMana
 	/* show the notification */
 	gpm_manager_notify (manager, &manager->priv->notification_discharging, title, message, GPM_MANAGER_NOTIFY_TIMEOUT_LONG,
 			    icon, NOTIFY_URGENCY_NORMAL);
+out:
 	g_free (icon);
-out:
+	g_free (remaining_text);
 	return;
 }
 
-/**
- * gpm_manager_sleep_failure_response_cb:
- **/
-static void
-gpm_manager_sleep_failure_response_cb (GtkDialog *dialog, gint response_id, GpmManager *manager)
-{
-	GdkScreen *screen;
-	GtkWidget *dialog_error;
-	GError *error = NULL;
-	gboolean ret;
-	gchar *uri = NULL;
-
-	/* user clicked the help button */
-	if (response_id == GTK_RESPONSE_HELP) {
-		uri = gconf_client_get_string (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED_URI, NULL);
-		screen = gdk_screen_get_default();
-		ret = gtk_show_uri (screen, uri, gtk_get_current_event_time (), &error);
-		if (!ret) {
-			dialog_error = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_OK,
-							       "Failed to show uri %s", error->message);
-			gtk_dialog_run (GTK_DIALOG (dialog_error));
-			g_error_free (error);
-		}
-	}
-
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	g_free (uri);
-}
-
-/**
- * gpm_manager_sleep_failure_cb:
- **/
-static void
-gpm_manager_sleep_failure_cb (GpmControl *control, GpmControlAction action, GpmManager *manager)
-{
-	gboolean show_sleep_failed;
-	gchar *message = NULL;
-	gchar *title = NULL;
-	gchar *uri = NULL;
-	const gchar *icon;
-	GtkWidget *dialog;
-
-	/* only show this if specified in gconf */
-	show_sleep_failed = gconf_client_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED, NULL);
-
-	egg_debug ("sleep failed");
-	gpm_manager_play (manager, GPM_MANAGER_SOUND_SUSPEND_ERROR, TRUE);
-
-	/* only emit if in GConf */
-	if (!show_sleep_failed)
-		goto out;
-
-	/* TRANSLATORS: window title: there was a problem putting the machine to sleep */
-	title = g_strdup_printf ("%s: %s", GPM_NAME, _("Sleep problem"));
-	if (action == GPM_CONTROL_ACTION_SUSPEND) {
-		/* TRANSLATORS: message text */
-		message = g_strdup_printf ("%s\n%s", _("Your computer failed to suspend."), _("Check the help file for common problems."));
-		icon = GPM_STOCK_SUSPEND;
-	} else {
-		/* TRANSLATORS: message text */
-		message = g_strdup_printf ("%s\n%s", _("Your computer failed to hibernate."), _("Check the help file for common problems."));
-		icon = GPM_STOCK_HIBERNATE;
-	}
-
-	/* show modal dialog */
-	dialog = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-						     GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
-						     "<span size='larger'><b>%s</b></span>", title);
-	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), "%s", message);
-
-	/* show a button? */
-	uri = gconf_client_get_string (manager->priv->conf, GPM_CONF_NOTIFY_SLEEP_FAILED_URI, NULL);
-	if (uri != NULL && uri[0] != '\0') {
-		/* TRANSLATORS: button text, visit the suspend help website */
-		gtk_dialog_add_button (GTK_DIALOG (dialog), _("Visit help page"), GTK_RESPONSE_HELP);
-	}
-
-	/* wait async for close */
-	gtk_widget_show (dialog);
-	g_signal_connect (dialog, "response", G_CALLBACK (gpm_manager_sleep_failure_response_cb), manager);
-out:
-	g_free (uri);
-	g_free (title);
-	g_free (message);
-}
-
 /**
  * gpm_manager_engine_just_laptop_battery:
  */
@@ -1339,7 +1312,7 @@ gpm_manager_engine_just_laptop_battery (GpmManager *manager)
 {
 	DkpDevice *device;
 	DkpDeviceType type;
-	const GPtrArray *array;
+	GPtrArray *array;
 	gboolean ret = TRUE;
 	guint i;
 
@@ -1354,7 +1327,7 @@ gpm_manager_engine_just_laptop_battery (GpmManager *manager)
 			break;
 		}
 	}
-
+	g_ptr_array_unref (array);
 	return ret;
 }
 
@@ -1451,16 +1424,6 @@ out:
 	g_free (message);
 }
 
-/**
- * gpm_manager_get_time_until_action_text:
- */
-static gchar *
-gpm_manager_get_time_until_action_text (GpmManager *manager)
-{
-	guint time_s = 0;
-	return gpm_get_timestring (time_s);
-}
-
 /**
  * gpm_manager_engine_charge_critical_cb:
  */
@@ -1472,7 +1435,6 @@ gpm_manager_engine_charge_critical_cb (GpmEngine *engine, DkpDevice *device, Gpm
 	gchar *remaining_text;
 	gchar *action;
 	gchar *icon = NULL;
-	gchar *time_text;
 	DkpDeviceType type;
 	gdouble percentage;
 	gint64 time_to_empty;
@@ -1507,7 +1469,6 @@ gpm_manager_engine_charge_critical_cb (GpmEngine *engine, DkpDevice *device, Gpm
 		}
 
 		remaining_text = gpm_get_timestring (time_to_empty);
-		time_text = gpm_manager_get_time_until_action_text (manager);
 
 		/* we have to do different warnings depending on the policy */
 		action = gconf_client_get_string (manager->priv->conf, GPM_CONF_ACTIONS_CRITICAL_BATT, NULL);
@@ -1520,20 +1481,19 @@ gpm_manager_engine_charge_critical_cb (GpmEngine *engine, DkpDevice *device, Gpm
 
 		} else if (policy == GPM_ACTION_POLICY_SUSPEND) {
 			/* TRANSLATORS: give the user a ultimatum */
-			message = g_strdup_printf (_("Computer will suspend in %s."), time_text);
+			message = g_strdup_printf (_("Computer will suspend in %s."), remaining_text);
 
 		} else if (policy == GPM_ACTION_POLICY_HIBERNATE) {
 			/* TRANSLATORS: give the user a ultimatum */
-			message = g_strdup_printf (_("Computer will hibernate in %s."), time_text);
+			message = g_strdup_printf (_("Computer will hibernate in %s."), remaining_text);
 
 		} else if (policy == GPM_ACTION_POLICY_SHUTDOWN) {
 			/* TRANSLATORS: give the user a ultimatum */
-			message = g_strdup_printf (_("Computer will shutdown in %s."), time_text);
+			message = g_strdup_printf (_("Computer will shutdown in %s."), remaining_text);
 		}
 
 		g_free (action);
 		g_free (remaining_text);
-		g_free (time_text);
 	} else if (type == DKP_DEVICE_TYPE_UPS) {
 		/* TRANSLATORS: the UPS is very low */
 		title = _("UPS critically low");
@@ -1793,7 +1753,7 @@ static void
 gpm_manager_init (GpmManager *manager)
 {
 	gboolean check_type_cpu;
-	guint timeout;
+	gint timeout;
 	DBusGConnection *connection;
 	GError *error = NULL;
 	guint version;
@@ -1860,8 +1820,10 @@ gpm_manager_init (GpmManager *manager)
 
 	/* try and start an interactive service */
 	manager->priv->screensaver = gpm_screensaver_new ();
+#if 0
 	g_signal_connect (manager->priv->screensaver, "auth-request",
 			  G_CALLBACK (gpm_manager_screensaver_auth_request_cb), manager);
+#endif
 
 	/* try an start an interactive service */
 	manager->priv->backlight = gpm_backlight_new ();
@@ -1888,17 +1850,9 @@ gpm_manager_init (GpmManager *manager)
 	/* use the control object */
 	egg_debug ("creating new control instance");
 	manager->priv->control = gpm_control_new ();
-	g_signal_connect (manager->priv->control, "sleep-failure",
-			  G_CALLBACK (gpm_manager_sleep_failure_cb), manager);
 
 	egg_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
-	g_signal_connect_object (G_OBJECT (manager->priv->tray_icon),
-				 "suspend", G_CALLBACK (gpm_manager_tray_icon_suspend),
-				 manager, G_CONNECT_SWAPPED);
-	g_signal_connect_object (G_OBJECT (manager->priv->tray_icon),
-				 "hibernate", G_CALLBACK (gpm_manager_tray_icon_hibernate),
-				 manager, G_CONNECT_SWAPPED);
 
 	/* keep a reference for the notifications */
 	manager->priv->status_icon = gpm_tray_icon_get_status_icon (manager->priv->tray_icon);
